{
*****************************************************************************
*  Name    : String Library                                                 *
*  Author  : David John Barker                                              *
*  Notice  : Copyright (c) 2006 Mecanique                                   *
*          : All Rights Reserved                                            *
*  Date    : 29/08/05                                                       *
*  Version : 1.0                                                            *
*  Notes   :                                                                *
*          :                                                                *
*****************************************************************************
}
module Str 

// local constants
const
   SPACE = " "

{
****************************************************************************
* Name    : Length                                                         *
* Purpose : Returns the length of a string                                 *
****************************************************************************
}
public function Length(pStr as string) as byte
   result = 0
   FSR0 = addressof(pStr)
   while POSTINC0 <> 0
      inc(result)
   wend
end function 
{
****************************************************************************
* Name    : InternalCopy (PRIVATE)                                         *
* Purpose : Copies a portion of a string (from pStart to pLength) into a   *
*         : destination string                                             *
****************************************************************************
}
sub InternalCopy(byref pDestStr, pStr as string,pStart as byte,pLength as byte = $FF) 
   dim Index, StrLength as byte 
   StrLength = Length(pStr)
   Index = 0
   FSR0 = addressof(pDestStr)
   FSR1 = addressof(pStr) + pStart
   while (Index < pLength) and (pStart < StrLength)
      POSTINC0 = POSTINC1
	   inc (Index)
      inc (pStart)
   end while   
   INDF0 = 0
end sub   
{
****************************************************************************
* Name    : Copy                                                           *
* Purpose : Copies a portion of a string (from pStart to pLength) and      *
*         : returns the result                                             *
****************************************************************************
}
public function Copy(pStr as string, pStart, pLength as byte) as string auto(pStr)
   InternalCopy(Result, pStr, pStart, pLength)
end function   
{
****************************************************************************
* Name    : Uppercase (OVERLOAD)                                           *
* Purpose : Returns a string that has been converted to uppercase. Only    *
*         : lowercase letters are converted to uppercase. All uppercase    *
*         : letters and non-letter characters remain unchanged.            *
****************************************************************************
}
public function Uppercase(pStr as string) as string auto(pStr)
   FSR0 = addressof(result)
   FSR1 = addressof(pStr)
asm    
   movlw 97
   subwf INDF1,W
   bn $ + 12
   movlw 123
   subwf INDF1,W
   bnn $ + 6
   movlw 223
   andwf INDF1,F
   movf  POSTINC1, W
   movwf POSTINC0
   bnz $ - 20
end asm
end function
{
****************************************************************************
* Name    : Uppercase (OVERLOAD)                                           *
* Purpose : Returns a single char that has been converted to uppercase.    *
*         : Only lowercase letters are converted to uppercase. All         *
*         : uppercase letters and non-letter characters remain unchanged.  *
****************************************************************************
}
public function Uppercase(pChar as char) as char
   if pChar >= "a" and pChar <= "z" then
      dec(pChar, 32)
   endif   
   result = pChar
end function
{
****************************************************************************
* Name    : Lowercase (OVERLOAD)                                           *
* Purpose : Returns a String that has been converted to lowercase. Only    *
*         : uppercase letters are converted to lowercase. All lowercase    *
*         : letters and non-letter characters remain unchanged.            *
****************************************************************************
}
public function Lowercase(pStr as string) as string auto(pStr)
   FSR0 = addressof(result)
   FSR1 = addressof(pStr)
asm    
   movlw 65
   subwf INDF1,W
   bn $ + 12
   movlw 91
   subwf INDF1,W
   bnn $ + 6
   movlw 32
   iorwf INDF1,F
   movf  POSTINC1, W
   movwf POSTINC0
   bnz $ - 20
end asm
end function
{
****************************************************************************
* Name    : Lowercase (OVERLOAD)                                           *
* Purpose : Returns a single char that has been converted to lowercase.    *
*         : Only uppercase letters are converted to lowercase. All         *
*         : lowercase letters and non-letter characters remain unchanged.  *
****************************************************************************
}
public function Lowercase(pChar as char) as char
   if pChar >= "A" and pChar <= "Z" then
      inc(pChar, 32)
   endif   
   result = pChar
end function
{
****************************************************************************
* Name    : Position                                                       *
* Purpose : Returns an integer specifying the position of the first        *
*         : occurrence of one string (sub-string) within another. If the   *
*         : sub-string is not found, -1 is returned                        *
****************************************************************************
}
public function Position(pSubStr,pStr as string) as integer
   dim Index, SubIndex, SubLength as byte

   result = -1
   SubIndex = 0
   Index = 0
   SubLength = Length(pSubStr)
   while (SubIndex < SubLength) and (pStr(Index) <> NULL)
      if pStr(Index) = pSubStr(SubIndex) then
         inc(SubIndex)
  	     if SubIndex = SubLength then
	        result = integer(Index) - SubLength + 1
         end if
      else
	     SubIndex = 0
      end if
	  inc(Index)
   end while
end function
{
****************************************************************************
* Name    : Mid                                                            *
* Purpose : Returns a string containing a specified number of characters   *
*         : from a string.                                                 *
****************************************************************************
}
public function Mid(pStr as string,pStart as byte,pLength as byte = $FF) as string
   InternalCopy(pStr,pStr,pStart,pLength)
   result = pStr
end function
{
****************************************************************************
* Name    : Left                                                           *
* Purpose : Returns a string containing a specified number of characters   *
*         : from the left side of a string.                                *                             
****************************************************************************
}
public function Left(pStr as string,pLength as byte) as string
   InternalCopy(pStr,pStr,0,pLength)
   result = pStr
end function
{
****************************************************************************
* Name    : Right                                                          *
* Purpose : Returns a string containing a specified number of characters   *
*         : from the right side of a string.                               *          
****************************************************************************
}
public function Right(pStr as string,pLength as byte) as string
   InternalCopy(pStr,pStr,Length(pStr) - pLength)
   result = pStr
end function
{
****************************************************************************
* Name    : Delete                                                         *
* Purpose : Returns a string containing a copy of the specified string,    *
*         : minus the first sub-string. If the sub-string is not found,    *
*         : the whole string is returned.                                  *
****************************************************************************
}
public function Delete(pSub, pStr as string) as string
   dim Pos as byte 
      
   result = pStr
   Pos = Position(pSub,pStr)
   if Pos <> $FF then  
      InternalCopy(result,result,0,Pos)
      InternalCopy(pStr,pStr,Pos + Length(pSub))
	  result = result + pStr
   end if   
end function
{
****************************************************************************
* Name    : Insert                                                         *
* Purpose : Merges source (pStr) into pDest at the position pIndex         *
****************************************************************************
}
public sub Insert(pStr as string,byref pDest as string,pIndex as byte)
   pDest = Copy(pDest,0, pIndex) + pStr + Copy(pDest,pIndex,$FF)
end sub
{
****************************************************************************
* Name    : TrimLeft                                                       *
* Purpose : Returns a string containing a copy of a specified string       *
*         : with all leading spaces removed.                               *
****************************************************************************
}
public function TrimLeft(pStr as string) as string
   dim Index, StrLength as byte
   
   Index = 0
   StrLength = Length(pStr)
   while (Index < StrLength) and (pStr(Index) = SPACE) 
      inc(index)
   end while 
   InternalCopy(pStr,pStr,Index) 
   result = pStr
end function
{
****************************************************************************
* Name    : TrimRight                                                      *
* Purpose : Returns a string containing a copy of a specified string       *
*         : with all trailing space removed                                *
****************************************************************************
}
public function TrimRight(pStr as string) as string
   dim Index as byte

   Index = Length(pStr)
   while (Index > 0) and (pStr(Index - 1) = SPACE) 
      dec(Index)
   end while
   InternalCopy(pStr,pStr,0,Index)
   result = pStr
end function
{
****************************************************************************
* Name    : Trim                                                           *
* Purpose : Returns a string containing a copy of a specified string       *
*         : with all leading and trailing spaces removed.                  *
****************************************************************************
}
public function Trim(pStr as string) as string
   result = TrimLeft(TrimRight(pStr))
end function
{
****************************************************************************
* Name    : Compare                                                        *
* Purpose : Compares two strings, returns...                               *
*         : < 0 string A is less than string B                             *
*         : = 0 string A is equal to string B                              *
*         : > 0 string A is greater than string B                          *
****************************************************************************
}
public function Compare(pStrA, pStrB as string) as shortint
   dim ValueA, ValueB as byte
   dim PtrA as POSTINC0
   dim PtrB as POSTINC1   
   FSR0 = @pStrA
   FSR1 = @pStrB
   Result = 0
   repeat
      ValueA = PtrA
      ValueB = PtrB
      Result = ValueA - ValueB
   until (Result <> 0) or (ValueA = 0) or (ValueB = 0)
end function


